import constants as cfg


class NLP(object):
    def __init__(self, parser, classifier):
        self.parser = parser
        self.clf = classifier
        self.x, self.y = None, None

    def parse_data_set(self):
        self.parser.parse()
        self.x = self.parser.get_column_data(cfg.COLUMN_FEATURE)
        self.y = self.parser.get_column_data(cfg.COLUMN_LABEL)

    def train(self):
        self.clf.fit(self.x, self.y)

    def predict(self, x):
        return self.clf.predict(x)





import unittest

import cerebro.neuron as neu


class TestFinder(unittest.TestCase):

    def test_find_neurons(self):
        neu.cfg.add_neurons_location(neu.STR_DEFAULT_NEURONS_PATH)
        self.assertIsNotNone(neu.find(neu.cfg.get_neurons_path()))



import unittest

import cerebro.nlp as n


class TestParser(unittest.TestCase):

    def setUp(self):
        self.obj = n.DataSetParser(n.DATA_SET_PATH)
        self.obj.parse()

    def test_parse(self):
        self.assertIsNotNone(self.obj.path)
        self.assertIsNotNone(self.obj.df)

    def test_column_data(self):
        temp = self.obj.get_column_data(n.COLUMN_LABEL)
        self.assertEqual(len(temp), 29)



import unittest
import numpy as np

import cerebro.nlp as n


class TestNLP(unittest.TestCase):

    def setUp(self):
        parser = n.DataSetParser(n.DATA_SET_PATH)
        classifier = n.CerebroClassifier()
        self.obj = n.NLP(parser, classifier)

    def test_init(self):
        self.assertIsNotNone(self.obj.parser)
        self.assertIsNotNone(self.obj.clf)
        self.assertIsNone(self.obj.x)
        self.assertIsNone(self.obj.y)

    def test_parse_data_set(self):
        self.obj.parse_data_set()
        self.assertEqual(len(self.obj.x), 29)
        self.assertEqual(len(self.obj.y), 29)

    def test_train(self):
        self.obj.parse_data_set()
        self.obj.train()

    def test_predict(self):
        self.obj.parse_data_set()
        self.obj.train()
        test_y = self.obj.predict(["How are you"])
        self.assertGreater(np.mean(test_y == ["greet"]), 0.5)



from datetime import datetime


def current_time(*args):
    now = datetime.now()
    return now.strftime("%H:%M")


def current_date(*args):
    now = datetime.now()
    return now.strftime("%d-%m-%Y")


KEYWORDS = {
    "time": current_time,
    "date": current_date
}


import unittest

import cerebro.data.neurons.simple as sim


class TestSimple(unittest.TestCase):

    def setUp(self):
        self.response = "All working properly."
        self.keyword = "greet"
        self.valid_attr = "KEYWORDS"

    def test_valid(self):
        self.assertTrue(hasattr(sim, self.valid_attr))

    def test_simple(self):
        self.assertEqual(sim.simple_test(), self.response)

    def test_neuron(self):
        self.assertEqual(sim.KEYWORDS[self.keyword](), self.response)



import unittest

import cerebro.neuron as neu


class TestConfig(unittest.TestCase):
    def test_config_creation(self):
        self.assertIsNotNone(neu.cfg.get_neurons_path())

    def test_add_path(self):
        neu.cfg.add_neurons_location(neu.STR_DEFAULT_NEURONS_PATH)
        self.assertNotEquals(len(neu.cfg.get_neurons_path()), 0)



import core.factory as default
from neuron.entities import Command

factory = default.DefaultFactory()
nlp, neuron = factory.cerebro()


def run(txt):
    label = nlp.predict(txt)
    return neuron.process(Command(label[0]))


import abc

import cerebro.neuron as neu
import cerebro.nlp as n


class BaseFactory(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def neuron(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def nlp(self):
        raise NotImplementedError()

    def cerebro(self):
        neuron = self.neuron()
        neuron.load_all()

        nlp = self.nlp()
        nlp.parse_data_set()
        nlp.train()

        return nlp, neuron


class DefaultFactory(BaseFactory):

    def neuron(self):
        neu.cfg.add_neurons_location(neu.STR_DEFAULT_NEURONS_PATH)
        finder = neu.find
        manager = neu.NeuronsManager()
        return neu.Neuron(neu.cfg, finder, manager)

    def nlp(self):
        parser = n.DataSetParser(n.DATA_SET_PATH)
        classifier = n.CerebroClassifier()
        return n.NLP(parser, classifier)



import unittest

import cerebro.neuron.neuron as neu
import cerebro.nlp.nlp as n
import cerebro.core.factory as fact


class TestFactory(unittest.TestCase):

    def setUp(self):
        self.obj = fact.DefaultFactory()

    def test_type(self):
        self.assertIsInstance(self.obj, fact.BaseFactory)

    def test_creation(self):
        self.assertIsNotNone(self.obj)

    def test_neuron(self):
        neuron = self.obj.neuron()
        self.assertIsInstance(neuron, neu.Neuron)

    def test_nlp(self):
        nlp = self.obj.nlp()
        self.assertIsInstance(nlp, n.NLP)

    def test_cerebro(self):
        nlp, neuron = self.obj.cerebro()
        self.assertIsInstance(nlp, n.NLP)
        self.assertIsInstance(neuron, neu.Neuron)


import unittest
import numpy as np

from cerebro.nlp.classifier import CerebroClassifier, AbstractClassifier


class TestClassifier(unittest.TestCase):

    def setUp(self):
        self.obj = CerebroClassifier()
        self.test_x = ["test1", "test2"]
        self.test_y = ["t1", "t2"]

    def test_type(self):
        self.assertIsInstance(self.obj, AbstractClassifier)

    def test_init(self):
        self.assertIsNotNone(self.obj.clf)

    def test_fit(self):
        self.obj.fit(self.test_x, self.test_y)
        self.assertGreater(np.mean(self.obj.predict(self.test_x) == self.test_y), 0.5)





import unittest
from datetime import datetime

import cerebro.data.neurons.datetime_util as dt


class TestDatetime(unittest.TestCase):

    def setUp(self):
        self.time_response = datetime.now().strftime("%H:%M")
        self.time_keyword = "time"
        self.date_response = datetime.now().strftime("%d-%m-%Y")
        self.date_keyword = "date"
        self.valid_attr = "KEYWORDS"

    def test_valid(self):
        self.assertTrue(hasattr(dt, self.valid_attr))

    def test_time(self):
        self.assertEqual(dt.current_time(), self.time_response)

    def test_neuron_time(self):
        self.assertEqual(dt.KEYWORDS[self.time_keyword](), self.time_response)

    def test_date(self):
        self.assertEqual(dt.current_date(), self.date_response)

    def test_neuron_date(self):
        self.assertEqual(dt.KEYWORDS[self.date_keyword](), self.date_response)



import unittest

import cerebro.neuron as neu


class TestNeuron(unittest.TestCase):
    def setUp(self):
        neu.cfg.add_neurons_location(neu.STR_DEFAULT_NEURONS_PATH)

        self.finder = neu.find

        self.manager = neu.NeuronsManager()

        self.command_args = ("arg1", "arg2")

    def test_neuron_creation(self):
        neuron = neu.Neuron(neu.cfg, self.finder, self.manager)
        self.assertIsNotNone(neuron)
        self.assertEqual(neuron.config, neu.cfg)
        self.assertEqual(neuron.finder, self.finder)
        self.assertEqual(neuron.manager, self.manager)

    def test_load_all(self):
        neuron = neu.Neuron(neu.cfg, self.finder, self.manager)
        neuron.load_all()
        self.assertGreater(len(self.manager.get_all()), 0)

    def test_command_execution(self):
        test_keyword = "greet"
        test_response = "All working properly."
        test_command = neu.Command(test_keyword, self.command_args)

        neuron = neu.Neuron(neu.cfg, self.finder, self.manager)
        neuron.load_all()

        response = neuron.process(test_command)
        self.assertEqual(response, test_response)

    def test_command_execution_failure(self):
        test_keyword = "wrong command"
        test_command = neu.Command(test_keyword)

        neuron = neu.Neuron(neu.cfg, self.finder, self.manager)
        neuron.load_all()

        response = neuron.process(test_command)
        self.assertEqual(response, neu.STR_DEFAULT_RESPONSE)



import abc

import constants as const


class AbstractManager(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def get_all(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def add(self, obj):
        raise NotImplementedError()

    @abc.abstractmethod
    def is_valid(self, neuron):
        raise NotImplementedError()

    @abc.abstractmethod
    def get(self, keyword):
        raise NotImplementedError()

    @abc.abstractmethod
    def is_exists(self, keyword):
        raise NotImplementedError()

    @abc.abstractmethod
    def execute(self, keyword, args=None):
        raise NotImplementedError()


class NeuronsManager(AbstractManager):
    def __init__(self):
        self.neurons = {}

    def get_all(self):
        return self.neurons

    def add(self, obj):
        self.neurons.update(obj)

    def is_valid(self, neuron):
        return hasattr(neuron, const.STR_KEYWORDS)

    def get(self, keyword):
        return self.neurons.get(keyword)

    def is_exists(self, keyword):
        return self.neurons.has_key(keyword)

    def execute(self, keyword, args=None):
        return self.get(keyword)(args) if self.is_exists(keyword) else const.STR_DEFAULT_RESPONSE






import os
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

STR_DEFAULT_RESPONSE = "Sorry, I could not process that."
STR_KEYWORDS = "KEYWORDS"


STR_DEFAULT_NEURONS_PATH = BASE_DIR + "/data/neurons"



import pkgutil


def find(path):
    for package in pkgutil.walk_packages(path):
        finder, name = package[0], package[1]
        loader = finder.find_module(name)
        mod = loader.load_module(name)
        yield mod



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open as opn
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with opn(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(name='lebot-cerebro',
      version='0.1.0dev7',
      description='Core engine for LeBot',
      long_description=long_description,
      classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
      ],
      keywords=['Chat Bot, AI'],
      url='https://github.com/Le-Bot/cerebro',
      author='LeBot',
      author_email='sanket.upadhyay@infoud.co.in',
      license='MIT',
      packages=['cerebro']+find_packages(),
      package_data={'': ['*.csv'], 'cerebro.data': ['datasets/*.csv']},
      install_requires=[
          'scikit-learn',
          'scipy',
          'numpy',
          'pandas',
      ],
      test_suite='nose.collector',
      tests_require=['nose'],
      include_package_data=True,
      zip_safe=False)



import abc

from sklearn.feature_extraction.text import CountVectorizer, TfidfTransformer
from sklearn.pipeline import Pipeline
from sklearn.linear_model import SGDClassifier


class AbstractClassifier(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def fit(self, features, labels):
        raise NotImplementedError()

    @abc.abstractmethod
    def predict(self, features):
        raise NotImplementedError()


class CerebroClassifier(AbstractClassifier):
    def __init__(self):
        self.clf = Pipeline([
            ('vect', CountVectorizer()),
            ('tfidf', TfidfTransformer()),
            ('clf', SGDClassifier(max_iter=5, tol=None)),
        ])

    def fit(self, features, labels):
        self.clf.fit(features, labels)

    def predict(self, features):
        return self.clf.predict(features)



import pandas as pd


class DataSetParser(object):
    def __init__(self, path):
        self.df = None
        self.path = path

    def parse(self):
        self.df = pd.read_csv(self.path)

    def get_column_data(self, name):
        # Retried features and Labels column
        return list(self.df[name])



import unittest

from cerebro.neuron.manager import AbstractManager, NeuronsManager


def simple_neuron(*args):
    return "This is dummy neuron"


class TestManager(unittest.TestCase):

    KEYWORDS = {"test": simple_neuron}

    def setUp(self):
        self.obj = NeuronsManager()
        self.obj.add(self.KEYWORDS)
        self.keyword = "test"

    def test_type(self):
        self.assertIsInstance(self.obj, AbstractManager)

    def test_neurons_creation(self):
        self.assertIsNotNone(self.obj.get_all())

    def test_add_neuron(self):
        self.assertEquals(len(self.obj.get_all()), 1)

    def test_is_valid_neuron(self):
        self.assertTrue(self.obj.is_valid(self))

    def test_get_neuron(self):
        self.assertIsNotNone(self.obj.get(self.keyword))

    def test_is_exists_neuron(self):
        self.assertTrue(self.obj.is_exists(self.keyword))

    def test_execute_neuron(self):
        self.assertEquals(self.obj.execute(self.keyword), "This is dummy neuron")






import unittest

import cerebro.neuron.entities as en


class TestEntities(unittest.TestCase):

    def setUp(self):
        self.keyword = "book"
        self.test_keyword = "book"
        self.args = ("arg2", "arg1")
        self.test_args = ("arg2", "arg1")

    def test_command_creation(self):
        command = en.Command(self.keyword)
        self.assertEquals(command.keyword, self.test_keyword)

    def test_command_creation_args(self):
        command = en.Command(self.keyword, self.args)
        self.assertEquals(command.keyword, self.test_keyword)
        self.assertEquals(command.args, self.test_args)



class Neuron(object):
    def __init__(self, config, finder, manager):
        self.config = config
        self.finder = finder
        self.manager = manager

    def load_all(self):
        path = self.config.get_neurons_path()
        neuron_modules = self.finder(path)
        for neuron in neuron_modules:
            if not self.manager.is_valid(neuron):
                continue

            self.manager.add(neuron.KEYWORDS)

    def process(self, command):
        return self.manager.execute(command.keyword, command.args)